from collections import OrderedDict


class Polynomial:
    def __init__(self, *elements):
        self.__representation = {}  # Python common dictionary...
        for e in elements:
            # print(e)
            self.__representation[e[1]] = e[0]  # Powers are keys...
        # sorted(self.__representation)  # Sort on keys (ascending)... returns a list...
        # Required for Horner eval.:
        self.__representation = OrderedDict(sorted(self.__representation.items(), key=lambda pair: pair[0],
                                                   reverse=True))  # Sort on keys (descending)...
        # print(type(self.__representation)) # '<class 'collections.OrderedDict'>' is displayed
        # print(self.__representation)

    def eval(self, x):
        result = 0
        for key, value in self.__representation.items():
            # print(key)
            result += pow(x, key) * value
        return result

    # https://en.wikipedia.org/wiki/Horner%27s_method
    def eval_Horner(self, x):  # Horner method that required less multiplication stuff...
        prior_key = None
        result = 0
        for key in self.__representation:
            if prior_key is not None:
                assert key < prior_key  # Imposed by sorting in '__init__'
                # print(key)
                result += self.__representation.get(prior_key)
                for i in range(prior_key, key, -1):
                    # print(i, end='-')
                    result *= x
            prior_key = key
        result += self.__representation.get(prior_key)
        for i in range(prior_key, 0, -1):
            # print(i, end='-')
            result *= x
        return result


p1 = Polynomial((5, 2), (-3, 1), (8, 0))
print('eval p1:', p1.eval(2))  # '22' is displayed
print('eval_Horner:', p1.eval_Horner(2))

p2 = Polynomial((-12, 10), (8, 39))
print('eval p2:', p2.eval(1))  # '-4' is displayed
print('eval_Horner:', p2.eval_Horner(1))

p3 = Polynomial((3, 11), (-8, 7), (-7, 1), (-43, 0))
print('eval p3:', p3.eval(5))  # '145859297' is displayed
print('eval_Horner:', p3.eval_Horner(5))
